from datetime import timedelta
import sqlalchemy as sa
from ringo.model.base import BaseItem, BaseFactory
from ringo.model.modul import ModulItem, _create_default_actions
from ringo.model.mixins import Owned, Meta, Nested
from plorma.model import Base
from plorma.model.timelog import RUNNING, PAUSED

nm_task_users = sa.Table(
    'nm_task_users', Base.metadata,
    sa.Column('uid', sa.Integer, sa.ForeignKey('users.id')),
    sa.Column('tid', sa.Integer, sa.ForeignKey('tasks.id'))
)


class TaskFactory(BaseFactory):

    def create(self, user=None):
        new_item = BaseFactory.create(self, user)
        return new_item


class Task(BaseItem, Meta, Nested, Owned, Base):
    __tablename__ = 'tasks'
    _modul_id = 9

    # Attributes
    id = sa.Column(sa.Integer, primary_key=True)
    account = sa.Column(sa.String)
    title = sa.Column(sa.String, nullable=False)
    description = sa.Column(sa.Text)
    type_id = sa.Column(sa.String, default="1")
    resolution_id = sa.Column(sa.String, default="1")
    status_id = sa.Column(sa.String, default="1")
    priority_id = sa.Column(sa.String, default="3")
    severity_id = sa.Column(sa.String, default="3")
    estimated_time = sa.Column(sa.Integer)
    deadline = sa.Column(sa.Date)

    # ForeignKeys
    assigned_id = sa.Column(sa.Integer, sa.ForeignKey('users.id'))

    # Relations
    nosy = sa.orm.relationship("User", secondary=nm_task_users)
    assigned = sa.orm.relationship("User",
                                   primaryjoin="User.id==Task.assigned_id")

    # Configuration
    _table_fields = [('title', 'Title'),
                     ('priority_id', 'Priority'), ('severity_id', 'Severity'),
                     ('status_id', 'Status'), ('resolution_id', 'Resolution')]


    def get_account(self, recursive=False):
        account = []
        if self.parent and recursive:
            account.append(self.parent.get_account(recursive))
        if not self.account:
            account.append("#%s" % self.id)
        else:
            account.append(self.account)
        return "-".join(account)

    def get_time(self, user=None, start=None, end=None):
        """Returns total timedelta of timelogs within the given start
        and end date. If no start date is given then start with the
        first available timelog. If no end date is given stop with the
        last timelog. If no user is given. sum up all timelogs for all
        users in the given timeframe.

        :user: User for which the timelogs will be summed.
        :start: Start date
        :end: End date
        :returns: seconds as integer

        """
        seconds = 0
        for timelog in self.timelogs:
            if (user is None) or (timelog.uid == user.id):
                if (((start is None) or (start <= timelog.updated))
                and ((end is None) or (end >= timelog.updated))):
                    seconds += timelog.seconds
            else:
                continue
        return timedelta(seconds=seconds)


    def get_status(self, user=None):
        """Returns the status of the newest timelog for the given user.
        If no user is given then the function returns the status of the newst timelog at all.
        If there is no timelog then return None.

        :user: User for which the status of this task is determined.
        :returns: Integer or None

        """
        last_timelog = self.get_last_timelog(user)
        if last_timelog:
            return last_timelog.status
        return None

    def is_running(self, user=None):
        """Returns True if the task is running for the given user.
        If no user is given then the function returns True if the task
        is running for any other user. A task is running if the
        newest attached timelog (per user) has the state "RUNNING".
        :returns: True or False
        """
        return self.get_status(user) in [RUNNING]

    def in_progress(self, user=None):
        """Returns True if the task is in progress for the given user.
        If no user is given then the function returns True if the task
        is in progress for any other user. A task in progress if the
        newest attached timelog (per user) is marked as active.
        :returns: True or False

        """
        return self.get_status(user) in [RUNNING, PAUSED]

    def get_last_timelog(self, user=None):
        """Returns the last created timelog for the user in this task.
        If user is None then the newest timelog entry is returned. If
        there is no timelog at all return None.

        :user: User the timelog belongs to
        :returns: Timelog or None

        """
        times = sorted(self.timelogs, key=lambda time: time.id, reverse=True)
        for time in times:
            if (user is None) or (time.uid == user.id):
                return time
        return None


def init_model(dbsession):
    """Will setup the initial model for the task.

    :dbsession: Database session to which the items will be added.
    :returns: None
    """
    modul = ModulItem(name='tasks')
    modul.clazzpath = "plorma.model.task.Task"
    modul.label = "Task"
    modul.label_plural = "Tasks"
    modul.display = "header-menu"
    modul.actions.extend(_create_default_actions(dbsession))
    dbsession.add(modul)
