from datetime import datetime
import sqlalchemy as sa
from ringo.model import Base
from ringo.model.base import BaseItem, BaseFactory
from ringo.model.modul import ModulItem, _create_default_actions
from ringo.model.mixins import Owned
from ringo.model.user import User

from plorma.model.timelog import Timelog, RUNNING, PAUSED, STOPPED


# NM-Table definitions
nm_workbench_tasks = sa.Table(
    'nm_workbench_tasks', Base.metadata,
    sa.Column('wid', sa.Integer, sa.ForeignKey('workbenchs.id')),
    sa.Column('tid', sa.Integer, sa.ForeignKey('tasks.id'))
)


class WorkbenchFactory(BaseFactory):

    def create(self, user=None):
        new_item = BaseFactory.create(self, user)
        return new_item


class Workbench(BaseItem, Owned, Base):
    __tablename__ = 'workbenchs'
    _modul_id = 12
    id = sa.Column(sa.Integer, primary_key=True)
    tasks = sa.orm.relationship("Task",
                                secondary=nm_workbench_tasks)
    user = sa.orm.relationship(User,
                               backref=sa.orm.backref("workbench", cascade="all,delete"),
                               single_parent=True,
                               uselist=False)

    def get_tasks(self):
        return self.tasks

    def add_task(self, task):
        self.tasks.append(task)

    def del_task(self, task):
        self.tasks.remove(task)

    def start_task(self, task, user):
        # Create a new timelog entry
        timelog = task.get_last_timelog(user)
        now = datetime.now()
        if timelog and timelog.status == PAUSED:
            # FIXME: handle updating updated attribute in the Meta mixin
            # (torsten) <2013-10-07 18:01>
            timelog.updated = now
            timelog.status = RUNNING
        else:
            factory = Timelog.get_item_factory()
            timelog = factory.create(user)
            # FIXME: handle updating updated attribute in the Meta mixin
            # (torsten) <2013-10-07 18:01>
            timelog.created = now
            timelog.updated = now
            timelog.status = RUNNING
            task.timelogs.append(timelog)

    def pause_task(self, task, user):
        timelog = task.get_last_timelog(user)
        if timelog and timelog.status == RUNNING:
            td = datetime.now() - timelog.updated
            timelog.seconds += td.total_seconds()
            # FIXME: handle updating updated attribute in the Meta mixin
            # (torsten) <2013-10-07 18:01>
            timelog.updated = datetime.now()
            timelog.status = PAUSED

    def stop_task(self, task, user):
        timelog = task.get_last_timelog(user)
        if timelog and timelog.status in [RUNNING, PAUSED]:
            if timelog.status == RUNNING:
                td = datetime.now() - timelog.updated
                timelog.seconds += td.total_seconds()
            # FIXME: handle updating updated attribute in the Meta
            # mixin (torsten) <2013-10-07 18:01>
            timelog.updated = datetime.now()
            timelog.status = STOPPED

    def __unicode__(self):
        return str(self.id)


def init_model(dbsession):
    """Will setup the initial model for the workbench.

    :dbsession: Database session to which the items will be added.
    :returns: None
    """
    modul = ModulItem(name='workbenchs')
    modul.clazzpath = "plorma.model.workbench.Workbench"
    modul.label = "Workbench"
    modul.label_plural = "Workbenches"
    modul.display = "hidden"
    modul.actions.extend(_create_default_actions(dbsession,
                                                 ignore=['create', 'delete']))
    dbsession.add(modul)
