/***************************************************************************
 *                Helpers for parsing the timelog datatable                *
 ***************************************************************************/

function getTableData (callback) {
    /* Will return the parsed values form the timelog datatable as list */
    var data = [];
    var rows = $("td.timelog");
    for (var i = rows.length - 1; i >= 0; i--){
        var row = parseZeiterfassung(rows[i].innerHTML);
        data.push(row);
    }
    if (callback) {
        callback(data);
    }
    return data;
}

function parseZeiterfassung(row) {
    /* */
    date_index = 0;
    time_index = 1;
    user_index = 4;
    account_index = 6;
    var tokens = row.replace( /\n/g, " " ).replace("&nbsp;", "").split( " " );
    var date = tokens[date_index];
    var user = tokens[user_index];
    var account = parseAccount(tokens[account_index]);
    var time =  convert2Minutes(tokens[time_index]);
    var prow = {"date": date, "seconds": time, "user": user, "account": account[0], "activity": account[1]};
    return prow;
}

function parseAccount (argument) {
    var regex = /\[(.*)-(\w{1})\]/;
    var account = regex.exec(argument)[1];
    var activity = regex.exec(argument)[2];
    return [account, activity];
}


function convert2Minutes (argument) {
    var x = argument.split("h");
    var z = x[0].split(":");
    var seconds = parseInt(z[0])*60 + parseInt(z[1]);
    return seconds;
}

/***************************************************************************
 * Helper function for rendering diagrams. Function beginning with "_" are *
 * not meant to be called directly.                                        *
 ***************************************************************************/


function _convertData (data) {
    /* Will convert the rows in the data into native datatypes if needed. So
     * datestrings will be actually converted into a date object. */
    for (var i = data.length - 1; i >= 0; i--){
        var d = data[i];
        data[i].date = d3.time.format("%d.%M.%Y").parse(d.date);
    }
    return data;
}

function _filterData (data, key, value) {
    /* Will return only the rows in data which have the given value for
     * the given key. */
    var filtered = [];
    for (var i = data.length - 1; i >= 0; i--) {
        if (data[i][key] === value) {
            filtered.push(data[i]);
        }
    }
    return filtered;
}

function _sumTime (data, key) {
    var result = {};
    var seconds = 0;
    for (var i = data.length - 1; i >= 0; i--) {
        seconds = seconds + data[i].seconds;
        result[key] = data[i][key];
        result['seconds'] = seconds;
    }
    return result;
}

function _getUniqueValues (data, key) {
    /* Returns a list of unique values in the given data for the given key */
    var processed = [];
    for (var i = data.length - 1; i >= 0; i--) {
        var value = data[i][key];
        if (processed.indexOf(value) == -1) {
            processed.push(value);
        }
    }
    return processed;
}

function getTotalTime(data) {
    var tmp = 0;
    var seconds = 0;
    var minutes = 0;
    var hours = 0
    for (var i = data.length - 1; i >= 0; i--) {
        tmp = tmp + data[i].seconds;
    }
    hours = Math.floor(tmp/60);
    minutes = tmp % 60;
    return sprintf("%02d", hours)+":"+sprintf("%02d", minutes)+":"+sprintf("%02d", seconds);
}

function group (data, key) {
    var uv = _getUniqueValues(data, key);
    var newdata = [];
    for (var i = uv.length - 1; i >= 0; i--) {
        newdata.push(_sumTime(_filterData(data, key, uv[i]), key));
    }
    return newdata;
}



function getMax (data, key) {
    /* Will return the maximum value for the given key in data */
    values = data.map(function(r) { return r[key]; });
    return d3.max(values);
}

function getMin (data, key) {
    /* Will return the minimum value for the given key in data */
    values = data.map(function(r) { return r[key]; });
    return d3.min(values);
}


/***************************************************************************
 *                                Diagrams                                 *
 ***************************************************************************/

function makeScatterDiagram (element, vdata) {

    var data = _convertData(vdata);

    var margin = {top: 20, right: 20, bottom: 30, left: 50};
    var width = $(element).width() - margin.left - margin.right;
    var height = 200 - margin.top - margin.bottom;

    var xfield = "date";
    var yfield = "seconds";

    var x = d3.time.scale()
        .domain([getMin(data, xfield), getMax(data, xfield)])
        .range([0, width]);

    var y = d3.scale.linear()
        .domain([0, getMax(data, yfield)])
        .range([height, 0]);

    var color = d3.scale.category10();

    var canvas = d3.select(element)
        .append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

    // Add X-Axis
    var xAxis = d3.svg.axis()
    .scale(x)
    .ticks(d3.time.day, 1)
    .orient("bottom");

    canvas.append("g")
    .attr("class", "x axis")
    .attr("transform", "translate(0," + height + ")")
    .call(xAxis);

    // Add Y-Axis
    var yAxis = d3.svg.axis()
    .scale(y)
    .orient("left");

    canvas.append("g")
    .attr("class", "y axis")
      .call(yAxis)
        .append("text")
        .attr("transform", "rotate(-90)")
        .attr("y", 6)
        .attr("dy", ".71em")
        .style("text-anchor", "end")
        .text("Time (m)");

    canvas.selectAll(".dot")
    .data(data)
    .enter()
        .append("circle")
        .attr("class", "dot")
        .attr("r", 3.5)
        .attr("cx", function(d) { return x(d.date); })
        .attr("cy", function(d) { return y(d.seconds); })
        .style("fill", function(d) { return color(d.account); });

    var legend = canvas.selectAll(".legend")
        .data(color.domain())
        .enter().append("g")
        .attr("class", "legend")
        .attr("transform", function(d, i) { return "translate(0," + i * 10 + ")"; });

    legend.append("rect")
    .attr("x", width - 9)
    .attr("width", 9)
    .attr("height", 9)
    .style("fill", color);

    legend.append("text")
    .attr("x", width - 12)
    .attr("y", 5)
    .attr("dy", ".35em")
    .style("text-anchor", "end")
    .text(function(d) { return d; });
}

function makePieDiagram(element, vdata, groupby) {

    var data = group(vdata, groupby);

    var margin = {top: 20, right: 20, bottom: 30, left: 50};
    var width = $(element).width() - margin.left - margin.right;
    var height = 200 - margin.top - margin.bottom;
    var radius = Math.min(width, height) / 2;

    var color = d3.scale.category10();

    var arc = d3.svg.arc()
    .outerRadius(radius - 10)
    .innerRadius(0);

    var pie = d3.layout.pie()
        .sort(null)
        .value(function(d) { return d.seconds; });

    var canvas = d3.select(element).append("svg")
        .attr("width", width)
        .attr("height", height)
        .append("g")
        .attr("transform", "translate(" + width / 2 + "," + height / 2 + ")");

    var g = canvas.selectAll(".arc")
        .data(pie(data))
        .enter().append("g")
        .attr("class", "arc");

    g.append("path")
    .attr("d", arc)
    .style("fill", function(d) { return color(d.data[groupby]); });

    g.append("text")
    .attr("transform", function(d) { return "translate(" + arc.centroid(d) + ")"; })
    .attr("dy", ".35em")
    .attr("class", "pie-text")
    .style("text-anchor", "middle")
    .text(function(d) { return d.data[groupby] });
}

function createLineDiagram (element, vdata) {

    var data = group(vdata, "date");
    var xfield = "date";
    var yfield = "seconds";

    var margin = {top: 20, right: 20, bottom: 30, left: 50};
    var width = $(element).width() - margin.left - margin.right;
    var height = 200 - margin.top - margin.bottom;

    var x = d3.time.scale()
        .domain([getMin(data, xfield), getMax(data, xfield)])
        .range([0, width]);

    var y = d3.scale.linear()
        .domain([0, getMax(data, yfield)])
        .range([height, 0]);

    var line = d3.svg.line()
        .x(function(d) {return x(d[xfield])})
        .y(function(d) {return y(d[yfield])})
        .interpolate("step-before")

    var canvas = d3.select(element)
        .append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

    // Add path
    canvas.append("path")
    .attr("d", line(data))
    .attr("class", "line")
    .attr("stroke-width", 1)

    // Add X-Axis
    var xAxis = d3.svg.axis()
    .scale(x)
    .ticks(data.length)
    .orient("bottom");

    canvas.append("g")
    .attr("class", "x axis")
    .attr("transform", "translate(0," + height + ")")
    .call(xAxis);

    // Add Y-Axis
    var yAxis = d3.svg.axis()
    .scale(y)
    .orient("left");

    canvas.append("g")
    .attr("class", "y axis")
      .call(yAxis)
        .append("text")
        .attr("transform", "rotate(-90)")
        .attr("y", 6)
        .attr("dy", ".71em")
        .style("text-anchor", "end")
        .text("Time (m)");
}
