import logging
from pyramid.view import view_config

from ringo.views.base import list_, create_, update_, read_, delete_
from ringo.views.json import (
    list_   as json_list,
    create_ as json_create,
    update_ as json_update,
    read_   as json_read,
    delete_ as json_delete
    )
from ringo.lib.message import Mailer, Mail
from ringo.lib.helpers import get_app_name
from plorma.model.task import Task
from plorma.model.comment import Comment
from plorma.views.comments import CommentRenderer

log = logging.getLogger(__name__)


def _send_notification_email(request, task, recipients):
    """Will send a notification email that the task has been changed to
    the given list of recipients.

    :request: Current Request
    :task: Task
    :recipients: List of recipients
    :returns: task

    """
    mailer = Mailer(request)
    if not mailer.enabled:
        return task

    _ = request.translate
    settings = request.registry.settings
    addresses = []
    for user in recipients:
        if user.id == request.user.id:
            continue  # Do not send the email to the current user
        email = user.profile[0].email
        if email:
            addresses.append(email)
        else:
            log.warning("Missing email for User '%s'. " % user
                        + "Abort sending notification mail.")
    # Build Changes. Note: This currently does not work.
    values = []
    for col in task.get_columns():
        values.append("%s: %s" % (col, task.get_value(col)))

    subject = _('[task%s] %s' % (task.id, task.title))
    values = {'url': request.route_url('tasks-read', id=task.id),
              'comment': task.comments[-1],
              'values': values,
              'app_name': get_app_name(),
              'task': task,
              'user': request.user,
              'email': settings['mail.default_sender'],
              '_': _}
    if addresses:
        mail = Mail(addresses, subject, template="task", values=values)
        mailer.send(mail)
    return task


def _add_comment_callback(request, task):
    """Will add a comment to the task if the users adds a new comment.
    Further add the user to the team if there is a new commet and the
    user is not already member of the team.

    :request: @todo
    :task: @todo
    :returns: @todo

    """
    # Check if need to add a new Comment
    user_comment = request.params.get('comment')
    if user_comment:
        factory = Comment.get_item_factory()
        comment = factory.create(request.user)
        comment.text = user_comment
        task.comments.append(comment)
        task = _add_team_member(request, task)
    return task


def _add_team_member(request, task):
    """Adds the given user to the given task if the user is not already
    member of the task.

    :task: Task
    :user: User
    :returns: Task

    """
    if request.user.id not in [u.id for u in task.nosy]:
        task.nosy.append(request.user)
    return task


def create_task_callback(request, task):
    """The call back will be call when a new task has been created.
    The callback will do the following things:

    * Add the current user into the list of team members if he is not
      already a member of the team.
    * Send a notification mail to all members of the tasks group.

    :request: Current request
    :task: New created Task
    :return Task:"""
    task = _add_team_member(request, task)
    task = _send_notification_email(request, task, task.group.members)
    return task


def update_task_callback(request, task):
    """The call back will be call when a task has been updated.
    The callback will do the following things:

    * Adds a new comment if needed.
    * Add the current user into the list of team members if he adds a
      new comment and is not already a member of the team.
    * Send a notification mail to all members of the team.

    :request: Current request
    :task: New created Task
    :return Task:"""
    task = _add_comment_callback(request, task)
    task = _send_notification_email(request, task, task.nosy)
    return task


@view_config(route_name=Task.get_action_routename('list'),
             renderer='/default/list.mako',
             permission='list')
def list(request):
    return list_(Task, request)


@view_config(route_name=Task.get_action_routename('create'),
             renderer='/default/create.mako',
             permission='create')
def create(request):
    return create_(Task, request, callback=create_task_callback)


@view_config(route_name=Task.get_action_routename('update'),
             renderer='/default/update.mako',
             permission='update')
def update(request):
    renderers = {}
    renderers['commentlist'] = CommentRenderer
    return update_(Task, request, callback=update_task_callback,
                   renderers=renderers)


@view_config(route_name=Task.get_action_routename('read'),
             renderer='/default/read.mako',
             permission='read')
def read(request):
    renderers = {}
    renderers['commentlist'] = CommentRenderer
    return read_(Task, request, renderers=renderers)


@view_config(route_name=Task.get_action_routename('delete'),
             renderer='/default/confirm.mako',
             permission='delete')
def delete(request):
    return delete_(Task, request)

###########################################################################
#                               REST SERVICE                              #
###########################################################################

@view_config(route_name=Task.get_action_routename('list', prefix="rest"),
             renderer='json',
             request_method="GET",
             permission='list'
             )
def rest_list(request):
    return json_list(Task, request)

@view_config(route_name=Task.get_action_routename('create', prefix="rest"),
             renderer='json',
             request_method="POST",
             permission='create')
def rest_create(request):
    return json_create(Task, request)

@view_config(route_name=Task.get_action_routename('read', prefix="rest"),
             renderer='json',
             request_method="GET",
             permission='read')
def rest_read(request):
    return json_read(Task, request)

@view_config(route_name=Task.get_action_routename('update', prefix="rest"),
             renderer='json',
             request_method="PUT",
             permission='update')
def rest_update(request):
    return json_update(Task, request)

@view_config(route_name=Task.get_action_routename('delete', prefix="rest"),
             renderer='json',
             request_method="DELETE",
             permission='delete')
def rest_delete(request):
    return json_delete(Task, request)
